
/*
 *  mtp is the ``Multi Tracked Paths'', an implementation of the
 *  k-shortest paths algorithm for multi-target tracking.
 *
 *  Copyright (c) 2012 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of mtp.
 *
 *  mtp is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  mtp is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with selector.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <iostream>
#include <fstream>
#include <sys/time.h>
#include <getopt.h>
#include <string.h>

using namespace std;

#include "mtp_tracker.h"

#define FILENAME_SIZE 1024

char trajectory_filename[FILENAME_SIZE];
char graph_filename[FILENAME_SIZE];
int verbose;

void usage(ostream *os) {
  (*os) << "mtp [-h|--help] [-v|--verbose] [-t|--trajectory-filename <trajectory filename>] [-g|--graph-filename <graph filename>] [<tracking parameter file>]" << endl;
  (*os) << endl;
  (*os) << "The mtp command processes a file containing the description of a topology" << endl;
  (*os) << "and detection scores, and prints the optimal set of trajectories." << endl;
  (*os) << "If no filename is provided, it will read the parameters from the stdin." << endl;
  (*os) << "If no trajectory filename is provided, it will write the result to" << endl;
  (*os) << "stdout." << endl;
  (*os) << endl;
  (*os) << "Written by Francois Fleuret. (C) Idiap Research Institute, 2012." << endl;
}

scalar_t diff_in_second(struct timeval *start, struct timeval *end) {
  return
    scalar_t(end->tv_sec - start->tv_sec) +
    scalar_t(end->tv_usec - start->tv_usec)/1000000;
}

void do_tracking(istream *in_tracker) {
  timeval start_time, end_time;
  MTPTracker *tracker = new MTPTracker();

  if(verbose) { cout << "Reading the tracking parameters." << endl; }
  tracker->read(in_tracker);

  if(verbose) {
    cout << "Building the graph ... "; cout.flush();
    gettimeofday(&start_time, 0);
  }
  tracker->build_graph();
  if(verbose) { 
    gettimeofday(&end_time, 0);
    cout << "done (" << diff_in_second(&start_time, &end_time) << "s)." << endl;
  }

  if(verbose) {
    cout << "Tracking ... "; cout.flush();
    gettimeofday(&start_time, 0);
  }
  tracker->track();
  if(verbose) {
    gettimeofday(&end_time, 0);
    cout << "done (" << diff_in_second(&start_time, &end_time) << "s)." << endl;
  }

  if(strcmp(trajectory_filename, "")) {
    ofstream out_traj(trajectory_filename);
    tracker->write_trajectories(&out_traj);
    if(verbose) { cout << "Wrote " << trajectory_filename << "." << endl; }
  } else {
    tracker->write_trajectories(&cout);
  }

  if(strcmp(graph_filename, "") != 0) {
    ofstream out_dot(graph_filename);
    tracker->print_graph_dot(&out_dot);
    if(verbose) { cout << "Wrote " << graph_filename << "." << endl; }
  }

  delete tracker;
}

static struct option long_options[] = {
  { "trajectory-file", 1, 0, 't' },
  { "graph-file", 1, 0, 'g' },
  { "help", no_argument, 0, 'h' },
  { "verbose", no_argument, 0, 'v' },
  { 0, 0, 0, 0 }
};

int main(int argc, char **argv) {
  int c;
  int error = 0, show_help = 0;

  strncpy(trajectory_filename, "", FILENAME_SIZE);
  strncpy(graph_filename, "", FILENAME_SIZE);
  verbose = 0;

  while ((c = getopt_long(argc, argv, "t:g:hv",
                          long_options, NULL)) != -1) {

    switch(c) {

    case 't':
      strncpy(trajectory_filename, optarg, FILENAME_SIZE);
      break;

    case 'g':
      strncpy(graph_filename, optarg, FILENAME_SIZE);
      break;

    case 'h':
      show_help = 1;
      break;

    case 'v':
      verbose = 1;
      break;

    default:
      error = 1;
      break;
    }
  }

  if(error) {
    usage(&cerr);
    exit(EXIT_FAILURE);
  }

  if(show_help) {
    usage(&cout);
    exit(EXIT_SUCCESS);
  }

  if(optind < argc) {
    ifstream *file_in_tracker = new ifstream(argv[optind]);
    if(file_in_tracker->good()) {
      do_tracking(file_in_tracker);
    } else {
      cerr << "Can not open " << argv[optind] << endl;
      exit(EXIT_FAILURE);
    }
    delete file_in_tracker;
  } else {
    do_tracking(&cin);
  }

  exit(EXIT_SUCCESS);
}
